/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LMDBAL_SERIALIZER_HPP
#define LMDBAL_SERIALIZER_HPP

#include "serializer.h"

template<class T>
LMDBAL::Serializer<T>::Serializer() :
    bytes(),
    buffer(&bytes),
    stream(&buffer)
{
    buffer.open(QIODevice::ReadWrite);
}

template<class T>
LMDBAL::Serializer<T>::Serializer(const T& value) :
    bytes(),
    buffer(&bytes),
    stream(&buffer)
{
    buffer.open(QIODevice::ReadWrite);
    _setData(value);
}

template<class T>
LMDBAL::Serializer<T>::~Serializer() {
    buffer.close();
}

template<class T>
MDB_val LMDBAL::Serializer<T>::setData(const T& value) {
    clear();
    _setData(value);
    return getData();
}

template<class T>
T LMDBAL::Serializer<T>::deserialize(const MDB_val& value) {
    T result;
    deserialize(value, result);

    return result;
}

template<class T>
void LMDBAL::Serializer<T>::deserialize(const MDB_val& value, T& result) {
    clear();
    bytes.setRawData((char*)value.mv_data, value.mv_size);
    stream >> result;
}

template<class T>
void LMDBAL::Serializer<T>::_setData(const T& value) {
    stream << value;
}

template<class T>
void LMDBAL::Serializer<T>::clear() {
    if (buffer.pos() > 0) {
        buffer.seek(0);
    }
}

template<class T>
MDB_val LMDBAL::Serializer<T>::getData() {
    MDB_val val;

    val.mv_size = buffer.pos();
    val.mv_data = (char*)bytes.data();

    return val;
}

#endif //LMDBAL_SERIALIZER_HPP
